
# ABOUT BLAZY
Provides integration with bLazy to lazy load and multi-serve images to save
bandwidth and server requests. The user will have faster load times and save
data usage if they don't browse the whole page.

## REQUIREMENTS
1. bLazy library:
   * [Download bLazy](https://github.com/dinbror/blazy)
   * Extract it as is, rename **blazy-master** to **blazy**, so the assets are:

      + **/sites/../libraries/blazy/blazy.min.js**

2. PHP 5.6+ (let us know if Blazy can go lower?)

3. Any of autoloader modules:
   * [registry_autoload](http://dgo.to/registry_autoload)
   * [xautoload](http://dgo.to/xautoload)
   * [autoload](http://dgo.to/autoload)
   * Others?

  Skip if you already one have, I meant, have one. If you have others, please
  create a feature to include it in the module `hook_requirements()`.

## INSTALLATION

### A sequential step below is critical otherwise potential errors:
1. Visit **/admin/modules** and install one of autoload modules:
   * [registry_autoload](http://dgo.to/registry_autoload)
   * [xautoload](http://dgo.to/xautoload)
   * [autoload](http://dgo.to/autoload)

   Save! Do not install Blazy, yet, since Blazy has no hard dependency on any.

2. Install Blazy. Once an autoloader is installed, it will be locked for
   Blazy usage later to avoid accidental removal.

As long as you stick to these 2 steps, it should be just fine like regular
modules with hard dependencies.

### Known issues:
* **autoload**: must run `drush aur` and `drush cc` on Blazy activation, or
  fatal. The same procedure applies whenever blazy-related modules are
  activated, or adding new classes. Especially during DEV, Alpha, Beta,
  before RC. If not using drush, consider the other two:

  **registry_autoload**, **xautoload**.

If any issue with other autoloaders, kindly let us know. Blazy doesn't have a
hard dependency on any, it is on your own discretion.

**At any rate, solution is available:**

* know how to run `drush cc all` or clear cache.
* Only **at worst** case, clear registry, or if you don't drush, install and
  know how to use registry_rebuild.module safely.

### <del>WTF</del> FTW?
Blazy uses classes as a direct backport of 8.x.

### More info:
[Drupal 7](http://drupal.org/documentation/install/modules-themes/modules-7)


## RECOMMENDED
* [Markdown](http://dgo.to/markdown)

  To make reading this README a breeze at [Blazy UI help](/admin/help/blazy_ui)


## FEATURES
* Supports core Image.
* Supports Picture.
* Supports Colorbox/ Photobox/ PhotoSwipe, also multimedia lightboxes.
* Multi-serving images for configurable breakpoints, almost similar to core
  Responsive image, only less complex.
* CSS background lazyloading, see Mason, GridStack, and Slick carousel.
* IFRAME urls via custom coded, via Media.
* Supports inline images and iframes with lightboxes, and grid or CSS3 Masonry
  via Blazy Filter. Enable Blazy filter at **/admin/config/content/formats**,
  and check out instruction at **/filter/tips**.
* Blazy Grid formatter for Image, Media and Text with multi-value.  
* Delay loading for below-fold images until 100px (configurable) before they are
  visible at viewport.
* A simple effortless CSS loading indicator.
* It doesn't take over all images, so it can be enabled as needed via Blazy
  formatter, or its supporting modules.


## OPTIONAL FEATURES
* Views fields for File Entity and Media integration, see Slick Browser.
* Views style plugin Blazy Grid for Grid Foundation or CSS3 Masonry.
* Field formatters: Blazy with Media integration.


## USAGES
Be sure to enable Blazy UI which can be uninstalled at production later.

* Go to Manage display page, e.g.:
  [Admin page displays](/admin/structure/types/manage/page/display)

* Find **Blazy** formatter under **Manage display**.

* Go to [Blazy UI](/admin/config/media/blazy) to manage few global options,
  including enabling support to bring Picture into blazy-related formatters.


## MODULES THAT INTEGRATE WITH OR REQUIRE BLAZY
* [Blazy PhotoSwipe](http://dgo.to/blazy_photoswipe)
* [GridStack](http://dgo.to/gridstack)
* [Intense](http://dgo.to/intense)
* [Mason](http://dgo.to/mason)
* [Slick](http://dgo.to/slick)
* [Slick Views](http://dgo.to/slick_views)
* [Slick Media](http://dgo.to/slick_media)
* [Slick Video](http://dgo.to/slick_video)
* [Slick Browser](http://dgo.to/slick_browser)
* [Jumper](http://dgo.to/jumper)

Most duplication efforts from the above modules will be merged into
\Drupal\blazy\Dejavu or anywhere else namespace.


## SIMILAR MODULES
[Lazyloader](https://www.drupal.org/project/lazyloader)


## CURRENT DEVELOPMENT STATUS
Not all Blazy 8.x features are ported, yet. It is a WIP like any Alpha.
Please stay optimistic that things are broken till we have a BETA, or RC.

A full release should be reasonable after proper feedbacks from the community,
some code cleanup, and optimization where needed. Patches are very much welcome.

Alpha, Beta, DEV releases are for developers only. Beware of possible breakage.


## UPDATE SOP:
Visit any of the following URLs when updating Blazy, or its related modules.
Please ignore any documentation if already aware of Drupal site building. This
is for the sake of completed documentation for those who may need it.

1. [Performance](/admin/config/development/performance)  
  Unless an update is required, clearing cache should fix most issues.
  * Hit **Clear all caches** button once the new Blazy in place.
  * Regenerate CSS and JS as the latest fixes may contain changes to the assets.
    Ignore below if you are aware, and found no asset changes from commits.
    Normally clearing cache suffices when no asset changes are found.
      * Uncheck CSS and JS aggregation options under Bandwidth optimization.
      * Save.
      * [Ignorable] See one of Blazy related pages if display is expected.
      * [Ignorable] Only clear cache if needed.
      * Check both options again.
      * Save again.
      * [Ignorable] Press F5, or CMD/ CTRL + R to refresh browser cache if
        needed.

2. [Admin status](/admin/reports/status)

  Check for any pending update, and run /update.php from the brower address bar.

3. If Twig templates are customized, compare against the latest.


## PROGRAMATICALLY
See blazy.api.php (WIP) for details.


## PERFORMANCE TIPS:
* If breakpoints provided with tons of images, using image styles with ANY crop
  is recommended to avoid image dimension calculation with individual images.
  The image dimensions will be set once, and inherited by all images as long as
  they contain word crop. If using scaled image styles, regular calculation
  applies.


## SUBMITTING PATCHES OR ISSUES
Please consider the following to help you explain better, and to help us
understand better your bug reports, or patches as needed:

1. [Issue summaries](https://www.drupal.org/issue-summaries)

2. [Issue template](https://www.drupal.org/node/1326662)

* If you hate formalism, consider a crystal clear line, or two in the body text.
* Avoid explaining everything in the title.
* Use body text for explanation purposes.
* If language is a barrier, use any available/ online translation tool.


### 1. SUBMITTING ISSUES
When submitting bug reports, please:

* be kind with proper reproduction, and enough details.  
* mention library version, related-module version, if any, active theme, or
  anything which may help us identify issue better.  
* ensure the library is loaded, not 404.  
* switch to stock Bartik for just in case it is your custom theme.  
* use matching or similar branches or tags for related modules.  
* check out dups.  
* file it a support request, if unsure. We'll mark a bug a bug even if you
  file it under support requests.  


### 2. SUBMITTING PATCHES
We consider a patch as help, they consider it a sale, so thank you in advanced!
In order for you to help, or buy, us successfully, please consider:

* communicating and filling out the body text with proper explanations, not in
  comments (unless for comment patches, of course).
  I've seen patches which broke a module, so explanation is a must.
  If you have no time to write it in the body text, please hold off till later!
* providing optional links to the change records, or docs, if any.
* providing links to docs is a must for coding standards issues.
  This also lets us, you and me, learn from the actual docs, not told by tools!
  We can just run `drupalcs ...`, but help is welcome, too, in case a miss.
* checking out the latest dev branch in case already resolved.
* providing reproduction steps for bug reports is a must. No repro, no bugs.

You must speak like human to human, and help us respect you, and your time.
Dumping patches with empty body text will be disregarded, till the above is met.

Thank you for your kind consideration, cooperation, and contribution!


## TROUBLESHOOTING AND KNOWN ISSUES
Resizing is not supported. Just reload the page.


### 1. VIEWS INTEGRATION
Blazy provides a simple Views field for File Entity, and Media.

When using Blazy formatter within Views, check **Use field template** under
**Style settings**, if trouble with Blazy Formatter as a stand alone Views
output.

On the contrary, uncheck **Use field template**, when Blazy formatter
is embedded inside another module such as Slick so to pass the renderable
array to work with accordingly.

This is a Views common gotcha with field formatter, so be aware of it.
If confusing, just toggle **Use field template**, and see the output. You'll
know which works.


### 2. BLAZY GRID WITH SINGLE VALUE FIELD
This is no issue at D8. Blazy Grid formatter is designed for multi-value fields.
Unfortunately no handy way to disable formatters for single value at D7. So
the formatter is available even for single value, but not actually
functioning. Please ignore it till we can get rid of it at D7, if possible,
without extra legs.

### 3. MIN-WIDTH
If the images appear to be shrink within a **floating** container, add
some expected width or min-width to the parent container via CSS accordingly.
Non-floating image parent containers aren't affected.

### 4. MIN-HEIGHT
Add a min-height CSS to individual element to avoid layout reflow if not using
**Aspect ratio** or when **Aspect ratio** is not supported such as with
Picture. Picture has its own. Otherwise some collapsed image containers will
defeat the purpose of lazyloading. When using CSS background, the container
may also be collapsed.

### 5. SOLUTIONS
Both layout reflow and lazyloading delay issues are actually taken care of
if **Aspect ratio** option is enabled in the first place.

Adjust, and override blazy CSS files accordingly.


## AUTHOR/MAINTAINER/CREDITS
gausarts

[Contributors](https://www.drupal.org/node/2663268/committers)


## READ MORE
See the project page on drupal.org:

[Blazy module](http://drupal.org/project/blazy)

See the bLazy docs at:

* [Blazy library](https://github.com/dinbror/blazy)
* [Blazy website](http://dinbror.dk/blazy/)
