<?php

namespace Drupal\blazy;

/**
 * Provides grid utilities.
 */
class BlazyGrid {

  /**
   * Returns items as a grid display wrapped by theme_item_list().
   *
   * @param array $items
   *   The grid items being modified.
   * @param array $settings
   *   The given settings.
   *
   * @return array
   *   The modified array of grid items.
   *
   * @todo implement cache.
   */
  public static function build(array $items = [], array $settings = []) {
    $settings += BlazyDefault::gridSettings();
    $blazy = empty($settings['blazy_data']) ? '' : drupal_json_encode($settings['blazy_data']);

    $grids = [];
    foreach ($items as $delta => $item) {
      // @todo: Support non-Blazy which normally uses item_id.
      if (is_array($item) && isset($item['#build']) && isset($item['#build']['settings'])) {
        $item['#build']['settings'] += $settings;
        $item['#build']['settings']['delta'] = $delta;
      }

      // Supports single formatter field or complex fields such as Views.
      // Views or entity_view may flattened $item into a string.
      $content = is_string($item) ? ['#markup' => $item] : ['content' => $item];
      $grid = Blazy::container($content, ['class' => 'grid__content']);
      $grids[$delta] = [
        'data' => drupal_render($grid),
        'class' => ['grid'],
      ];
    }

    // @todo #cache element.
    $count = empty($settings['count']) ? count($grids) : $settings['count'];
    $element = [
      '#theme' => 'item_list',
      '#items' => $grids,
      '#context' => ['settings' => $settings],
      '#attributes' => [
        'class' => [
          'blazy',
          'blazy--grid',
          'block-' . $settings['style'],
          'block-count-' . $count,
        ],
        'data-blazy' => $blazy,
      ],
    ];

    $attributes = &$element['#attributes'];
    if (!empty($settings['media_switch'])) {
      $switch = str_replace('_', '-', $settings['media_switch']);
      $attributes['data-' . $switch . '-gallery'] = TRUE;
      $attributes['class'][] = 'blazy--' . $switch;
    }

    $settings['grid_large'] = $settings['grid'];
    foreach (['small', 'medium', 'large'] as $grid) {
      if (!empty($settings['grid_' . $grid])) {
        $attributes['class'][] = $grid . '-block-' . $settings['style'] . '-' . $settings['grid_' . $grid];
      }
    }

    return $element;
  }

}
