<?php

namespace Drupal\blazy\Plugin\views\style;

use Drupal\blazy\BlazyDefault;
use Drupal\blazy\BlazyGrid;
use Drupal\blazy\BlazyManagerTrait;
use Drupal\blazy\Form\BlazyAdminTrait;
use views_plugin_style;

/**
 * Blazy style plugin.
 */
class BlazyViews extends views_plugin_style {

  use BlazyManagerTrait;
  use BlazyAdminTrait;

  /**
   * {@inheritdoc}
   */
  public function option_definition() {
    $options = parent::option_definition();
    foreach (BlazyDefault::gridSettings() as $key => $value) {
      $options[$key] = ['default' => $value];
    }
    return $options;
  }

  /**
   * Overrides StylePluginBase::buildOptionsForm().
   */
  public function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $definition = [
      'namespace'     => 'blazy',
      'forms'         => ['grid' => TRUE],
      'settings'      => $this->options,
      'style'         => TRUE,
      'grid_required' => TRUE,
      'opening_class' => 'form--views',
    ];

    // Build the form.
    $this->admin()->openingForm($form, $definition);
    $this->admin()->gridForm($form, $definition);

    if (isset($form['grid'])) {
      $form['grid']['#description'] = t('The amount of block grid columns for large monitors 64.063em.');
    }

    $this->admin()->finalizeForm($form, $definition);

    // Blazy doesn't need complex grid with multiple groups.
    unset($form['layout'], $form['preserve_keys'], $form['grid_header'], $form['visible_items'], $form['style']['#empty_option'], $form['grid']['#empty_option']);
  }

  /**
   * Overrides StylePluginBase::render().
   */
  public function render() {
    $settings                      = &$this->options;
    $settings['count']             = count($this->view->result);
    $settings['current_view_mode'] = $this->view->current_display;
    $settings['item_id']           = 'content';
    $settings['namespace']         = 'blazy';
    $settings['view_name']         = $this->view->name;

    $elements = [];
    foreach ($this->render_grouping($this->view->result, $settings['grouping']) as $rows) {
      $items = [];
      foreach ($rows as $index => $row) {
        $this->view->row_index = $index;

        $items[$index] = $this->view->style_plugin->row_plugin->render($row);
      }

      // Supports Blazy formatter multi-breakpoint images if available.
      // @todo useless at D7, as the output is always flattened.
      $item = isset($items[0]) ? $items[0] : NULL;
      if ($item && is_array($item)) {
        $this->manager()->isBlazy($settings, $item);
      }

      $elements = BlazyGrid::build($items, $settings);
      $elements['#attached'] = $this->manager()->attach($settings);
      unset($this->view->row_index);
    }

    return $elements;
  }

}
